@@ -25,28 +25,20 @@
  *
  */
 
+#include "os.h"
+
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-
-//SLM
-#ifdef WIN32
-#include <io.h>
-#define STDOUT_FILENO fileno(stdout)
-#else
 #include <unistd.h>
-#endif
-
 #include <ctype.h>
 #include <errno.h>
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
 
-//SLM
-#if defined(__bsdi__) || defined(__FreeBSD__) || defined WIN32
-typedef off_t off64_t;
+#if defined(SYS_UNIX) || defined(COMP_MSC) || defined(__APPLE__)
 #define lseek64 lseek
 #endif
 
@@ -210,6 +202,12 @@ struct VAL names_strf_auds[] = {
     { INT32,  "av_bps" },
     { INT16,  "blockalign" },
     { INT16,  "bits" },
+    { INT16,  "cbSize" },
+    { INT16,  "wID" },
+    { INT32,  "fdwFlags" },
+    { INT16,  "nBlockSize" },
+    { INT16,  "nFramesPerBlock" },
+    { INT16,  "nCodecDelay" },
     { EoLST,  NULL }    
 };
 
@@ -740,7 +738,7 @@ int AVI_scan(char *file_name)
 
     DWORD  chunksize;    /* size of the RIFF chunk data */
 
-#ifdef WIN32
+#if defined(SYS_WINDOWS)
     if (!(fd=open(file_name, O_RDONLY|O_BINARY))) {
 #else
     if (!(fd=open(file_name, O_RDONLY))) {
@@ -779,7 +777,7 @@ int AVI_dump(char *file_name, int mode)
 
     DWORD  chunksize;    /* size of the RIFF chunk data */
 
-#ifdef WIN32
+#if defined(SYS_WINDOWS)
     if (!(fd=open(file_name,O_RDONLY|O_BINARY))) return(1);
 #else
     if (!(fd=open(file_name,O_RDONLY))) return(1);
