@@ -25,22 +25,18 @@
  *
  */
 
+#include "os.h"
+
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <stdio.h>
 #include <fcntl.h>
 
-//SLM
-#ifdef __CYGWIN__
-#include <sys/types.h>
-#elif defined WIN32
-#include <stdint.h>
-#else
+#if defined(COMP_MSC)
 #include <unistd.h>
 #include <inttypes.h>
 #endif
 
-
 #include <limits.h>
 #include <stdlib.h>
 #include <string.h>
@@ -65,6 +61,66 @@ typedef struct
    off_t tot;
 } audio_index_entry;
 
+
+// Index types
+
+
+#define AVI_INDEX_OF_INDEXES 0x00             // when each entry in aIndex
+                                              // array points to an index chunk
+#define AVI_INDEX_OF_CHUNKS  0x01             // when each entry in aIndex
+                                              // array points to a chunk in the file
+#define AVI_INDEX_IS_DATA    0x80             // when each entry is aIndex is
+                                              // really the data
+// bIndexSubtype codes for INDEX_OF_CHUNKS
+//
+#define AVI_INDEX_2FIELD     0x01             // when fields within frames
+                                              // are also indexed
+
+
+
+typedef struct _avisuperindex_entry {
+    unsigned long long qwOffset;           // absolute file offset
+    unsigned long dwSize;                  // size of index chunk at this offset
+    unsigned long dwDuration;              // time span in stream ticks
+} avisuperindex_entry;
+
+typedef struct _avistdindex_entry {
+    unsigned long dwOffset;                // qwBaseOffset + this is absolute file offset
+    unsigned long dwSize;                  // bit 31 is set if this is NOT a keyframe
+} avistdindex_entry;
+
+
+// Base Index Form 'indx'
+typedef struct _avisuperindex_chunk {
+    char           fcc[4];
+    unsigned long  dwSize;                 // size of this chunk
+    unsigned short wLongsPerEntry;         // size of each entry in aIndex array (must be 8 for us)
+    unsigned char  bIndexSubType;          // future use. must be 0
+    unsigned char  bIndexType;             // one of AVI_INDEX_* codes
+    unsigned long  nEntriesInUse;          // index of first unused member in aIndex array
+    char           dwChunkId[4];           // fcc of what is indexed
+    unsigned long  dwReserved[3];          // meaning differs for each index type/subtype.
+                                           // 0 if unused
+    avisuperindex_entry *aIndex;
+} avisuperindex_chunk;
+    
+// Standard index 
+typedef struct _avistdindex_chunk {
+    char           fcc[4];                 // ix##
+    unsigned long  dwSize;                 // size of this chunk
+    unsigned short wLongsPerEntry;         // must be sizeof(aIndex[0])/sizeof(DWORD)
+    unsigned char  bIndexSubType;          // must be 0
+    unsigned char  bIndexType;             // must be AVI_INDEX_OF_CHUNKS
+    unsigned long  nEntriesInUse;          //
+    char           dwChunkId[4];           // '##dc' or '##db' or '##wb' etc..
+    unsigned long long qwBaseOffset;       // all dwOffsets in aIndex array are relative to this
+    unsigned long  dwReserved3;            // must be 0
+    avistdindex_entry *aIndex;
+} avistdindex_chunk;
+    
+
+
+
 typedef struct track_s
 {
 
@@ -73,6 +129,7 @@ typedef struct track_s
     long   a_rate;            /* Rate in Hz */
     long   a_bits;            /* bits per audio sample */
     long   mp3rate;           /* mp3 bitrate kbs*/
+    long   a_vbr;             /* 0 == no Variable BitRate */
 
     long   audio_strn;        /* Audio stream number */
     off_t  audio_bytes;       /* Total number of bytes of audio data */
@@ -82,10 +139,11 @@ typedef struct track_s
     long   audio_posc;        /* Audio position: chunk */
     long   audio_posb;        /* Audio position: byte within chunk */
  
-    off_t a_codech_off;       /* absolut offset of audio codec information */ 
-    off_t a_codecf_off;       /* absolut offset of audio codec information */ 
+    off_t  a_codech_off;       /* absolut offset of audio codec information */ 
+    off_t  a_codecf_off;       /* absolut offset of audio codec information */ 
 
     audio_index_entry *audio_index;
+    avisuperindex_chunk *audio_superindex;
 
 } track_t;
 
@@ -102,7 +160,13 @@ typedef struct
   uint32_t  bi_y_pels_per_meter;
   uint32_t  bi_clr_used;
   uint32_t  bi_clr_important;
-} BITMAPINFOHEADER;
+} alBITMAPINFOHEADER;
+
+#if __GNUC__ == 2
+#pragma pack(1)
+#else
+#pragma pack(push,1)
+#endif
 
 typedef struct
 {
@@ -113,7 +177,13 @@ typedef struct
   uint16_t  n_block_align;
   uint16_t  w_bits_per_sample;
   uint16_t  cb_size;
-} WAVEFORMATEX;
+} alWAVEFORMATEX;
+
+#if __GNUC__ == 2
+#pragma pack(4)
+#else
+#pragma pack(pop)
+#endif
 
 typedef struct
 {
@@ -138,7 +208,7 @@ typedef struct
   uint32_t dw_edit_count;
   uint32_t dw_format_change_count;
   char     sz_name[64];
-} AVISTREAMINFO;
+} alAVISTREAMINFO;
 
 typedef struct
 {
@@ -161,26 +231,31 @@ typedef struct
   
   track_t track[AVI_MAX_TRACKS];  // up to AVI_MAX_TRACKS audio tracks supported
   
-  off_t pos;        /* position in file */
+  off_t  pos;               /* position in file */
   long   n_idx;             /* number of index entries actually filled */
   long   max_idx;           /* number of index entries actually allocated */
   
-  off_t v_codech_off;       /* absolut offset of video codec (strh) info */ 
-  off_t v_codecf_off;       /* absolut offset of video codec (strf) info */ 
+  off_t  v_codech_off;      /* absolut offset of video codec (strh) info */ 
+  off_t  v_codecf_off;      /* absolut offset of video codec (strf) info */ 
   
   unsigned char (*idx)[16]; /* index entries (AVI idx1 tag) */
+
   video_index_entry *video_index;
+  avisuperindex_chunk *video_superindex;  /* index of indices */
+  int is_opendml;           /* set to 1 if this is an odml file with multiple index chunks */
   
-  off_t last_pos;          /* Position of last frame written */
+  off_t  last_pos;          /* Position of last frame written */
   unsigned long last_len;          /* Length of last frame written */
   int must_use_index;              /* Flag if frames are duplicated */
-  off_t movi_start;
+  off_t  movi_start;
   
   int anum;            // total number of audio tracks 
   int aptr;            // current audio working track 
+  int comment_fd;      // Read avi header comments from this fd
+  char *index_file;    // read the avi index from this file
   
-  BITMAPINFOHEADER *bitmap_info_header;
-  WAVEFORMATEX *wave_format_ex[AVI_MAX_TRACKS];
+  alBITMAPINFOHEADER *bitmap_info_header;
+  alWAVEFORMATEX *wave_format_ex[AVI_MAX_TRACKS];
 } avi_t;
 
 #define AVI_MODE_WRITE  0
@@ -262,8 +337,11 @@ int  AVI_close(avi_t *AVI);
 long AVI_bytes_written(avi_t *AVI);
 
 avi_t *AVI_open_input_file(char *filename, int getIndex);
+avi_t *AVI_open_input_indexfile(char *filename, int getIndex, char *indexfile);
 avi_t *AVI_open_fd(int fd, int getIndex);
+avi_t *AVI_open_indexfd(int fd, int getIndex, char *indexfile);
 int avi_parse_input_file(avi_t *AVI, int getIndex);
+int avi_parse_index_from_file(avi_t *AVI, char *filename);
 long AVI_audio_mp3rate(avi_t *AVI);
 long AVI_video_frames(avi_t *AVI);
 int  AVI_video_width(avi_t *AVI);
@@ -277,6 +355,7 @@ int  AVI_audio_format(avi_t *AVI);
 long AVI_audio_rate(avi_t *AVI);
 long AVI_audio_bytes(avi_t *AVI);
 long AVI_audio_chunks(avi_t *AVI);
+int  AVI_can_read_audio(avi_t *AVI);
 
 long AVI_max_video_chunk(avi_t *AVI);
 
@@ -290,6 +369,9 @@ long AVI_read_frame(avi_t *AVI, char *vidbuf, int *keyframe);
 int  AVI_set_audio_position(avi_t *AVI, long byte);
 int  AVI_set_audio_bitrate(avi_t *AVI, long bitrate);
 
+long AVI_get_audio_position_index(avi_t *AVI);
+int  AVI_set_audio_position_index(avi_t *AVI, long indexpos);
+
 long AVI_read_audio(avi_t *AVI, char *audbuf, long bytes);
 long AVI_read_audio_chunk(avi_t *AVI, char *audbuf);
 
@@ -320,6 +402,11 @@ int AVI_set_audio_track(avi_t *AVI, int track);
 int AVI_get_audio_track(avi_t *AVI);
 int AVI_audio_tracks(avi_t *AVI);
 
+void AVI_set_audio_vbr(avi_t *AVI, long is_vbr);
+long AVI_get_audio_vbr(avi_t *AVI);
+
+void AVI_set_comment_fd(avi_t *AVI, int fd);
+int  AVI_get_comment_fd(avi_t *AVI);
 
 struct riff_struct 
 {
@@ -353,6 +440,11 @@ struct wave_header
 	struct chunk_struct  data;
 };
 
+// Simple WAV IO
+int AVI_read_wave_header( int fd, struct wave_header * wave );
+int AVI_write_wave_header( int fd, const struct wave_header * wave );
+size_t AVI_read_wave_pcm_data( int fd, void * buffer, size_t buflen );
+size_t AVI_write_wave_pcm_data( int fd, const void * buffer, size_t buflen );
 
 
 struct AVIStreamHeader {
