@@ -11,9 +11,6 @@
 #include <errno.h>
 #include <fcntl.h>
 #include <limits.h>
-#if !defined(__bsdi__) && !defined(__FreeBSD__)
-#include <malloc.h>
-#endif // BSD
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
