@@ -10,9 +10,6 @@
 
 #include <errno.h>
 #include <fcntl.h>
-#if !defined(__bsdi__) && !defined(__FreeBSD__)
-#include <malloc.h>
-#endif // BSD
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -240,7 +237,7 @@ void handle_packet(stream_t *stream, ogg_packet *pack, ogg_page *page) {
       }
 
       sub = (char *)&pack->packet[hdrlen + 1];
-      if ((strlen(sub) > 1) || (*sub != ' ')) {
+      if ((strlen(sub) > 1) || ((strlen(sub) > 0) && (*sub != ' '))) {
         sst = (pack->granulepos / stream->sample_rate) * 1000;
         pgp = sst + lenbytes;
         sprintf(out, "%d\r\n%02d:%02d:%02d,%03d --> " \
@@ -428,22 +425,23 @@ void process_ogm(int fdin)
                     "unknown type.", __FILE__, numstreams + 1);
         }
       }  else if (((*pack.packet & PACKET_TYPE_BITS ) == PACKET_TYPE_HEADER) &&
-	          (pack.bytes >= (int)(sizeof(stream_header) + 1))) {
-        stream_header *sth = (stream_header *)(pack.packet + 1);
-        if (!strncmp(sth->streamtype, "video", 5)) {
+	          (pack.bytes >= (int)(sizeof(old_stream_header) + 1))) {
+        stream_header sth;
+        copy_headers(&sth, (old_stream_header *)&pack.packet[1], pack.bytes);
+        if (!strncmp(sth.streamtype, "video", 5)) {
           unsigned long codec;
           char ccodec[5];
-          strncpy(ccodec, sth->subtype, 4);
+          strncpy(ccodec, sth.subtype, 4);
           ccodec[4] = 0;
-          codec = (sth->subtype[0] << 24) + 
-            (sth->subtype[1] << 16) + (sth->subtype[2] << 8) + sth->subtype[3];
+          codec = (sth.subtype[0] << 24) + 
+            (sth.subtype[1] << 16) + (sth.subtype[2] << 8) + sth.subtype[3];
           if (verbose > 0)
             fprintf(stdout, "(%s) (v%d/%d) fps: %.3f width height: %dx%d " \
                     "codec: %p (%s)\n", __FILE__, nvstreams + 1,
                     numstreams + 1,
-                    (double)10000000 / (double)get_uint64(&sth->time_unit),
-                    get_uint32(&sth->sh.video.width),
-                    get_uint32(&sth->sh.video.height), (void *)codec,
+                    (double)10000000 / (double)get_uint64(&sth.time_unit),
+                    get_uint32(&sth.sh.video.width),
+                    get_uint32(&sth.sh.video.height), (void *)codec,
                     ccodec);
           stream = (stream_t *)malloc(sizeof(stream_t));
           if (stream == NULL) {
@@ -453,7 +451,7 @@ void process_ogm(int fdin)
           stream->stype = 'v';
           stream->serial = sno;
           stream->sample_rate = (double)10000000 /
-            (double)get_uint64(&sth->time_unit);
+            (double)get_uint64(&sth.time_unit);
           stream->sno = nvstreams + 1;
           memcpy(&stream->instate, &sstate, sizeof(sstate));
           if (extraction_requested(xvideo, nvstreams + 1, NOVIDEO)) {
@@ -471,8 +469,8 @@ void process_ogm(int fdin)
                         __FILE__, new_name, AVI_strerror());
                 exit(1);
               }
-              AVI_set_video(stream->avi, get_uint32(&sth->sh.video.width),
-                            get_uint32(&sth->sh.video.height),
+              AVI_set_video(stream->avi, get_uint32(&sth.sh.video.width),
+                            get_uint32(&sth.sh.video.height),
                             stream->sample_rate, ccodec);
             } else {
               sprintf(new_name, "%s-v%d.raw", basename, nvstreams + 1);
@@ -494,10 +492,10 @@ void process_ogm(int fdin)
           add_stream(stream);
           nvstreams++;
           numstreams++;
-        } else if (!strncmp(sth->streamtype, "audio", 5)) {
+        } else if (!strncmp(sth.streamtype, "audio", 5)) {
           int codec;
           char buf[5];
-          memcpy(buf, sth->subtype, 4);
+          memcpy(buf, sth.subtype, 4);
           buf[4] = 0;
           codec = strtoul(buf, NULL, 16);
           if (verbose > 0) {
@@ -507,12 +505,12 @@ void process_ogm(int fdin)
                     codec == ACPCM ? "PCM" : codec == 55 ? "MP3" :
                     codec == ACMP3 ? "MP3" :
                     codec == ACAC3 ? "AC3" : "unknown",
-                    get_uint16(&sth->bits_per_sample),
-                    get_uint16(&sth->sh.audio.channels),
-                    get_uint64(&sth->samples_per_unit));
+                    get_uint16(&sth.bits_per_sample),
+                    get_uint16(&sth.sh.audio.channels),
+                    get_uint64(&sth.samples_per_unit));
             fprintf(stdout, " avgbytespersec: %d blockalign: %hd\n",
-                    get_uint32(&sth->sh.audio.avgbytespersec),
-                    get_uint16(&sth->sh.audio.blockalign));
+                    get_uint32(&sth.sh.audio.avgbytespersec),
+                    get_uint16(&sth.sh.audio.blockalign));
           }
           stream = (stream_t *)malloc(sizeof(stream_t));
           if (stream == NULL) {
@@ -521,8 +519,8 @@ void process_ogm(int fdin)
           }
           stream->sno = nastreams + 1;
           stream->stype = 'a';
-          stream->sample_rate = get_uint64(&sth->samples_per_unit) *
-                                get_uint16(&sth->sh.audio.channels);
+          stream->sample_rate = get_uint64(&sth.samples_per_unit) *
+                                get_uint16(&sth.sh.audio.channels);
           stream->serial = sno;
           stream->acodec = codec;
           strncpy(stream->wh.riff.id, "RIFF", 4);
@@ -531,16 +529,16 @@ void process_ogm(int fdin)
           put_uint32(&stream->wh.format.len, 16);
           put_uint16(&stream->wh.common.wFormatTag, 1);
           put_uint16(&stream->wh.common.wChannels,
-                     get_uint16(&sth->sh.audio.channels));
+                     get_uint16(&sth.sh.audio.channels));
           put_uint32(&stream->wh.common.dwSamplesPerSec,
-                     get_uint64(&sth->samples_per_unit));
+                     get_uint64(&sth.samples_per_unit));
           put_uint32(&stream->wh.common.dwAvgBytesPerSec,
-                     get_uint16(&sth->sh.audio.channels) *
-                     get_uint64(&sth->samples_per_unit) *
-                     get_uint16(&sth->bits_per_sample) / 8);
+                     get_uint16(&sth.sh.audio.channels) *
+                     get_uint64(&sth.samples_per_unit) *
+                     get_uint16(&sth.bits_per_sample) / 8);
           put_uint16(&stream->wh.common.wBlockAlign, 4);
           put_uint16(&stream->wh.common.wBitsPerSample,
-                     get_uint16(&sth->bits_per_sample));
+                     get_uint16(&sth.bits_per_sample));
           strncpy(stream->wh.data.id, "data", 4);
           memcpy(&stream->instate, &sstate, sizeof(sstate));
           if (extraction_requested(xaudio, nastreams + 1, NOAUDIO)) {
@@ -576,7 +574,7 @@ void process_ogm(int fdin)
           add_stream(stream);
           nastreams++;
           numstreams++;
-        } else if (!strncmp(sth->streamtype, "text", 4)) {
+        } else if (!strncmp(sth.streamtype, "text", 4)) {
           if (verbose > 0)
             fprintf(stdout, "(%s) (t%d/%d) text/subtitle stream\n", __FILE__,
                     ntstreams + 1, numstreams + 1);
@@ -588,7 +586,7 @@ void process_ogm(int fdin)
           stream->sno = ntstreams + 1;
           stream->stype = 't';
           stream->sample_rate = (double)10000000 /
-            (double)get_uint64(&sth->time_unit);
+            (double)get_uint64(&sth.time_unit);
           stream->serial = sno;
           memcpy(&stream->instate, &sstate, sizeof(sstate));
           if (extraction_requested(xtext, ntstreams + 1, NOTEXT)) {
