@@ -67,6 +67,8 @@ FILE *out;
 filelist_t *input;
 int verbose;
 int force_flushing = 0;
+int omit_empty_packets = 0;
+int old_headers = 0;
 
 float video_fps = -1.0;
 
@@ -145,6 +147,9 @@ static void usage(void)
     "                           'LANGUAGE=English' or 'TITLE=Ally McBeal'.\n"
     "  -f, --fourcc <FOURCC>    Forces the FourCC to the specified value.\n"
     "                           Works only for video streams.\n"
+    "  --omit-empty-packets     Omit the empty subtitle packets.\n"
+    "  --old-headers            Assume OGM files have been created with\n"
+    "                           an older version of ogmmerge ( < 1.1).\n"
     "\n"
     " Other options:\n"
     "  -l, --list-types         Lists supported input file types.\n"
@@ -620,6 +625,10 @@ static void parse_args(int argc, char **argv)
       fprintf(stdout, "WARNING: You're enabling flushing after each OGG "
               "packet. This will take more space and is generally a bad "
               "idea. It's intended for debugging purposes only.\n");
+    } else if (!strcmp(argv[i], "--omit-empty-packets"))
+      omit_empty_packets = true;
+    else if (!strcmp(argv[i], "--old-headers")) {
+      old_headers = 1;
     }
 #ifdef ENABLE_INDEX
     else if (!strcmp(argv[i], "-i") || !strcmp(argv[i], "--index"))
