@@ -15,9 +15,6 @@
   or visit http://www.gnu.org/copyleft/gpl.html
 */
 
-#if !defined(__bsdi__) && !defined(__FreeBSD__)
-#include <malloc.h>
-#endif // BSD
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -267,7 +264,7 @@ void ogm_reader_c::handle_new_stream(ogg_page *og) {
   ogg_stream_state  new_oss;
   ogg_packet        op;
   ogm_demuxer_t    *dmx;
-  stream_header    *sth;
+  stream_header     sth;
   char             *codec;
 
   if (ogg_stream_init(&new_oss, ogg_page_serialno(og))) {
@@ -329,9 +326,9 @@ void ogm_reader_c::handle_new_stream(ogg_page *og) {
 
   // The new stream headers introduced by OggDS (see ogmstreams.h).
   if (((*op.packet & PACKET_TYPE_BITS ) == PACKET_TYPE_HEADER) &&
-      (op.bytes >= ((int)sizeof(stream_header) + 1))) {
-    sth = (stream_header *)(op.packet + 1);
-    if (!strncmp(sth->streamtype, "video", 5)) {
+      (op.bytes >= ((int)sizeof(old_stream_header) + 1))) {
+    copy_headers(&sth, (old_stream_header *)&op.packet[1], op.bytes);
+    if (!strncmp(sth.streamtype, "video", 5)) {
       nvstreams++;
       numstreams++;
       if (!demuxing_requested(vstreams, nvstreams)) {
@@ -344,17 +341,17 @@ void ogm_reader_c::handle_new_stream(ogg_page *og) {
         die("malloc");
       memset(dmx, 0, sizeof(ogm_demuxer_t));
       if (fourcc == NULL)
-        codec = sth->subtype;
+        codec = sth.subtype;
       else
         codec = fourcc;
       try {
         dmx->packetizer =
           new video_packetizer_c(codec, (double)10000000 /
-                                 (double)get_uint64(&sth->time_unit), 
-                                 get_uint32(&sth->sh.video.width),
-                                 get_uint32(&sth->sh.video.height),
-                                 get_uint16(&sth->bits_per_sample),
-                                 get_uint32(&sth->buffersize), NULL,
+                                 (double)get_uint64(&sth.time_unit), 
+                                 get_uint32(&sth.sh.video.width),
+                                 get_uint32(&sth.sh.video.height),
+                                 get_uint16(&sth.bits_per_sample),
+                                 get_uint32(&sth.buffersize), NULL,
                                  &range, comments);
       } catch (error_c error) {
         fprintf(stderr, "FATAL: ogm_reader: could not initialize video " \
@@ -371,7 +368,7 @@ void ogm_reader_c::handle_new_stream(ogg_page *og) {
       dmx->sid = nvstreams;
       add_new_demuxer(dmx);
       if (video_fps < 0)
-        video_fps = 10000000.0 / (double)get_uint64(&sth->time_unit);
+        video_fps = 10000000.0 / (double)get_uint64(&sth.time_unit);
       if (verbose)
         fprintf(stdout, "OGG/OGM demultiplexer (%s): using video output " \
                   "module for stream %d.\n", filename, numstreams);
@@ -379,7 +376,7 @@ void ogm_reader_c::handle_new_stream(ogg_page *og) {
       return;
     }
     
-    if (!strncmp(sth->streamtype, "audio", 5)) {
+    if (!strncmp(sth.streamtype, "audio", 5)) {
       char buf[5];
       long codec_id;
       
@@ -390,7 +387,7 @@ void ogm_reader_c::handle_new_stream(ogg_page *og) {
         return;
       }
       
-      memcpy(buf, (char *)sth->subtype, 4);
+      memcpy(buf, (char *)sth.subtype, 4);
       buf[4] = 0;
       codec_id = strtol(buf, (char **)NULL, 16);
       
@@ -402,9 +399,9 @@ void ogm_reader_c::handle_new_stream(ogg_page *og) {
         case 0x0001: // raw PCM
           try {
             dmx->packetizer =
-              new pcm_packetizer_c(get_uint64(&sth->samples_per_unit),
-                                   get_uint16(&sth->sh.audio.channels), 
-                                   get_uint16(&sth->bits_per_sample),
+              new pcm_packetizer_c(get_uint64(&sth.samples_per_unit),
+                                   get_uint16(&sth.sh.audio.channels), 
+                                   get_uint16(&sth.bits_per_sample),
                                    &async, &range,
                                    comments);
           } catch (error_c error) {
@@ -423,9 +420,9 @@ void ogm_reader_c::handle_new_stream(ogg_page *og) {
         case 0x0055: // MP3
           try {
             dmx->packetizer = 
-              new mp3_packetizer_c(get_uint64(&sth->samples_per_unit),
-                                   get_uint16(&sth->sh.audio.channels),
-                                   get_uint32(&sth->sh.audio.avgbytespersec) *
+              new mp3_packetizer_c(get_uint64(&sth.samples_per_unit),
+                                   get_uint16(&sth.sh.audio.channels),
+                                   get_uint32(&sth.sh.audio.avgbytespersec) *
                                    8 / 1000, &async, &range, comments);
           } catch (error_c error) {
             fprintf(stderr, "FATAL: ogm_reader: could not initialize MP3 " \
@@ -443,9 +440,9 @@ void ogm_reader_c::handle_new_stream(ogg_page *og) {
         case 0x2000: // AC3
           try {
             dmx->packetizer = 
-              new ac3_packetizer_c(get_uint64(&sth->samples_per_unit),
-                                   get_uint16(&sth->sh.audio.channels),
-                                   get_uint32(&sth->sh.audio.avgbytespersec) *
+              new ac3_packetizer_c(get_uint64(&sth.samples_per_unit),
+                                   get_uint16(&sth.sh.audio.channels),
+                                   get_uint32(&sth.sh.audio.avgbytespersec) *
                                    8 / 1000, &async, &range, comments);
           } catch (error_c error) {
             fprintf(stderr, "FATAL: ogm_reader: could not initialize AC3 " \
@@ -477,7 +474,7 @@ void ogm_reader_c::handle_new_stream(ogg_page *og) {
       return;
     }
 
-    if (!strncmp(sth->streamtype, "text", 4)) {
+    if (!strncmp(sth.streamtype, "text", 4)) {
 
       ntstreams++;
       numstreams++;
